<?php

namespace backend\controllers;

use Yii;
use backend\models\MainCategory;
use backend\models\MainCategorySearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use backend\models\Vendor;
use yii\filters\AccessControl;

use yii\web\UploadedFile;

/**
 * MainCategoryController implements the CRUD actions for MainCategory model.
 */
class MainCategoryController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['create', 'view','index','update'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
        ];
    }

    /**
     * Lists all MainCategory models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new MainCategorySearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    /**
     * Displays a single MainCategory model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        return $this->render('view', [
            'model' => $this->findModel($id),
        ]);
    }

    /**
     * Creates a new MainCategory model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $image_path="";
        $model = new MainCategory();
     

        if (Yii::$app->request->isPost) {
            $model->file = UploadedFile::getInstance($model, 'file');
            if(!$model->file){

                if ($model->load(Yii::$app->request->post()) ) {

                    $model->status=1;
                    $model->created_by=Yii::$app->getUser()->id;
                    $model->created_on=date('Y-m-d h:i:s');
                    $model->rating=1;
                    $model->image_path=$image_path;

                    $model->save();
                   return $this->redirect(['view', 'id' => $model->id]);
               }

               }
            //    need to add session vendor id
               $vendor_details=Vendor::find()->Where(['id'=>2])->one();


                $model->file->saveAs('../image/'.$vendor_details->database_name.'/' . $model->file->baseName . '.' . $model->file->extension);

                //save databases
               $image_path='../image/'.$vendor_details->database_name.'/' . $model->file->baseName . '.' . $model->file->extension; 

              
                if ($model->load(Yii::$app->request->post()) ) {

                     $model->status=1;
                     $model->created_by=Yii::$app->getUser()->id;
                     $model->created_on=date('Y-m-d h:i:s');
                     $model->rating=1;
                     $model->image_path=$image_path;

                     $model->save();
                    return $this->redirect(['view', 'id' => $model->id]);
                }
   
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing MainCategory model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing MainCategory model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the MainCategory model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return MainCategory the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = MainCategory::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }
}
