<?php

namespace backend\controllers;

use Yii;
use backend\models\PriceList;
use backend\models\PriceListSearch;
use backend\models\Notification;
use backend\models\PriceChangeNotice;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\ContentNegotiator;
use yii\web\Response;
use yii\helpers\Url;
use yii\filters\AccessControl;

/**
 * PriceListController implements the CRUD actions for PriceList model.
 */
class PriceListController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                    'changeapprove'=>['POST'],
                    'changereject'=>['POST']
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['create', 'view','index','update','changeapprove','requestprice','changereject','responsview'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'contentNegotiator' => [
                'class' => ContentNegotiator::className(),
                'only' => ['changeapprove','changereject'], 
                'formats' => [
                    'application/json' => Response::FORMAT_JSON,
                ],
            ]
        ];
    }

    /**
     * Lists all PriceList models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new PriceListSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }

    public function actionChangeapprove() { 

        $session = Yii::$app->session;            
        $notification_id = ($_POST['notification_id']);
        $pricelist_id = ($_POST['pricelist_id']);
        $product_id = ($_POST['product_id']);
        $description = ($_POST['description']);

        // return $description;

        $ModeNotification = Notification::find()->where(['id'=>$notification_id])->one();

        $ModeNotification->status=1;
        $ModeNotification->approved_on=date('Y-m-d h:i:s');
        $ModeNotification->approved_by=Yii::$app->user->identity->id;

        if($ModeNotification->save()){
            $model_pricelist = PriceList::find()->where(['id'=>$pricelist_id])->one();
            $model_pricelist2 = PriceList::find()->where(['product_id'=>$product_id, 'is_active' => 1])->one();
            
            $model_pricelist2->updated_date= date('Y-m-d h:i:s');
            $model_pricelist2->save();


            PriceList::updateAll(['is_active' => 0], ['=', 'product_id', $product_id]);

            
            $model_pricelist->is_active = 1;
           
           
            if($model_pricelist->save()){

                $modelPriceChangeNotice =new PriceChangeNotice();

                $modelPriceChangeNotice->status=1;
                $modelPriceChangeNotice->description=$description;
                $modelPriceChangeNotice->product_id=$product_id;
                $modelPriceChangeNotice->price_list_id=$model_pricelist->id;
                $modelPriceChangeNotice->user_id=Yii::$app->user->identity->id;
                $modelPriceChangeNotice->created_date=date('Y-m-d h:i:s');
                $modelPriceChangeNotice->save();
               

            





                return $this->redirect(['product/view', 'id' =>$product_id ]);
           
                
            }

        }
       return $notification_id;
    } 

    public function actionChangereject() { 

        $session = Yii::$app->session;            
        $notification_id = ($_POST['notification_id']);
        $pricelist_id = ($_POST['pricelist_id']);
        $product_id = ($_POST['product_id']);
        $description = ($_POST['description']);

        // return $description;

        $ModeNotification = Notification::find()->where(['id'=>$notification_id])->one();

        $ModeNotification->status=2;
        $ModeNotification->approved_on=date('Y-m-d h:i:s');
        $ModeNotification->approved_by=Yii::$app->user->identity->id;

        if($ModeNotification->save()){
            $model_pricelist = PriceList::find()->where(['id'=>$pricelist_id])->one();
            // $model_pricelist2 = PriceList::find()->where(['product_id'=>$product_id, 'is_active' => 1])->one();
            
            // $model_pricelist2->updated_date= date('Y-m-d h:i:s');
            // $model_pricelist2->save();


            // PriceList::updateAll(['is_active' => 0], ['=', 'product_id', $product_id]);

            
            $model_pricelist->is_active = 2;
           
           
            if($model_pricelist->save()){

                $modelPriceChangeNotice =new PriceChangeNotice();

                $modelPriceChangeNotice->status=-1;
                $modelPriceChangeNotice->description=$description;
                $modelPriceChangeNotice->product_id=$product_id;
                $modelPriceChangeNotice->price_list_id=$model_pricelist->id;
                $modelPriceChangeNotice->user_id=Yii::$app->user->identity->id;
                $modelPriceChangeNotice->created_date=date('Y-m-d h:i:s');
                $modelPriceChangeNotice->save();
            
                return $this->redirect(['product/view', 'id' =>$product_id ]);
           
                
            }

        }
       return $notification_id;
    } 

    /**
     * Displays a single PriceList model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        $request_pricelist = PriceList::find()->where(['id'=>$id,'is_active' => -1])->one();
        $current_pricelist = PriceList::find()->where(['product_id'=>$id , 'is_active' => 1])->one();

        // print_r($current_pricelist);
        // die;

        return $this->render('view', [
            'model' => $this->findModel($id),
            'current_pricelist'=>$current_pricelist
        ]);
    }
    public function actionRequestprice($id , $notif_id)
    {
        $request_pricelist = PriceList::find()->where(['id'=>$id,'is_active' => -1])->one();
        $current_pricelist = PriceList::find()->where(['product_id'=>$request_pricelist->product_id , 'is_active' => 1])->one();

        // print_r($current_pricelist);
        // die;

        return $this->render('requestprice', [
            'model' => $this->findModel($id),
            'current_pricelist'=>$current_pricelist,    
            'notif_id'=>$notif_id
        ]);
    }

    public function actionResponsview($id , $notif_id)
    {
        $modelNotification=Notification::find()->where(['id'=>$notif_id])->one();
        $request_pricelist = PriceList::find()->where(['id'=>$id])->one();

        $current_pricelist = PriceList::find()->where(['product_id'=>$request_pricelist->product_id , 'is_active' => 1])->one();

         $modelPriceChangeNotice = PriceChangeNotice::find()->where(['price_list_id'=>$id])->one();

        // $request_pricelist = PriceList::find()->where(['id'=>$id,'is_active' => -1])->one();
        // $current_pricelist = PriceList::find()->where(['product_id'=>$request_pricelist->product_id , 'is_active' => 1])->one();

        // print_r($current_pricelist);
        // die;

        return $this->render('responsview',[
            'model' => $this->findModel($id),
            'current_pricelist'=>$current_pricelist,    
            'modelNotification'=>$modelNotification,
            'notif_id'=>$notif_id,
            'modelPriceChangeNotice'=>$modelPriceChangeNotice



        ]);
    }


    /**
     * Creates a new PriceList model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new PriceList();

        if ($model->load(Yii::$app->request->post()) ) {

             $model->created_date=date('Y-m-d h:i:s');
             $model->save();



            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing PriceList model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);
        
        if ($model->load(Yii::$app->request->post()) ) {

            PriceList::updateAll(['is_active' => 0], ['=', 'id', $id]);

            $priceList = new PriceList;
            $priceList->description = $model->description;
            $priceList->is_active = 1;
            $priceList->product_id = $model->product_id;
            $priceList->selling_price = $model->selling_price;
            $priceList->buying_price = $model->buying_price;
            $priceList->created_date=date('Y-m-d h:i:s');
            $priceList->save();
            
             return $this->redirect(['view', 'id' => $priceList->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing PriceList model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the PriceList model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return PriceList the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = PriceList::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function beforeAction($action) 
    { 
        $this->enableCsrfValidation = false; 
        return parent::beforeAction($action); 
    } 
}

