<?php
namespace backend\controllers;

use Yii;
use yii\web\Controller;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use backend\models\User;
use backend\models\Vendor;
use backend\models\Order;
use backend\models\OrderItem;
use backend\models\PayTypes;
use backend\models\Product;
use yii\filters\ContentNegotiator;
use yii\web\Response;

use common\models\LoginForm;
use morozovsk\websocket\examples\chat3\server\Chat3WebsocketDaemonHandler;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;

/**
 * Site controller
 */
class SiteController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['login', 'error','order-view','orderdelivered','orderdetails','all-order-view','sales-report-exl','order-notification'],
                        'allow' => true,
                    ],
                    [
                        'actions' => ['logout', 'index','sales-report','set-order-type','moving-food','sales-report-exl','sales-product-report','sales-product-report-exl'],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                    [
                        'actions' => ['order-view'],
                        'allow' => true,
                        'roles' => ['*'],
                    ],
                ],
            ],
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'logout' => ['post'],
                ],
            ],
            'contentNegotiator' => [
                'class' => ContentNegotiator::className(),
                'only' => ['set-order-type','orderdelivered','orderdetails','all-order-view','order-notification'], 
                'formats' => [
                    'application/json' => Response::FORMAT_JSON,
                ],
            ]

            // 'ghost-access'=> [
            //     'class' => 'webvimark\modules\UserManagement\components\GhostAccessControl',
            // ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function actions()
    {
        return [
            'error' => [
                'class' => 'yii\web\ErrorAction',
            ],
        ];
    }

    /**
     * Displays homepage.
     *
     * @return string
     */

    public function setDb($database_name) 
    { 
 
        $connection = new \yii\db\Connection([
            'dsn' => 'mysql:host=localhost;dbname='.$database_name,
            'username' => 'eimuser',
            'password' => 'Welcome@2019',
           
            'charset' => 'utf8',
            ]);
            $connection->open();
         return $connection;
    } 
    public function actionIndex()
    {


  
        
        $allorderlit=[];
      $allCount=0;
      $allVendorOrderCount=0;
        $countuser=User::find()->where(['status'=>10])->count();
        $countSystemUser=User::find()->where(['status'=>1])->count();
        $countVendor= Vendor::find()->count();
        $allVendor= Vendor::find()->all();
        $OrderCount=Order::find()->where(['status'=>1])->count();

        foreach ($allVendor as $key => $allVendor) {  
                    $con = $this->setDb($allVendor->database_name);

                    $vendorViseOrder = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                 //   ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                    ->andWhere(['status'=>1])
                 //   ->andFilterWhere(['pay_type_id'=>$paymethod_id])

                    ->one($con);


                    $vendorViseOrderDaily = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                    ->andWhere(['status'=>1])
                    ->andWhere(['like', 'created_on', date('Y-m-d') . '%', false])
                    ->one($con);

                    $webOrderDaily = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                    ->andWhere(['status'=>1])
                    ->andWhere(['order_option_id'=>1])
                    ->andWhere(['like', 'created_on', date('Y-m-d') . '%', false])
                    ->one($con);
                        

                    

                    $mobileOrderDaily = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                    ->andWhere(['status'=>1])
                    ->andWhere(['order_option_id'=>2])
                   ->andWhere(['like', 'created_on', date('Y-m-d') . '%', false])
                    ->one($con);
                    
                    $posOrderDaily = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                    ->andWhere(['status'=>1])
                    ->andWhere(['order_option_id'=>3])
                    ->andWhere(['like', 'created_on', date('Y-m-d') . '%', false])
                    ->one($con);


                    $kioskOrderDaily = (new \yii\db\Query())
                    ->select('COUNT(*) as count')
                    ->from('order')
                    ->andWhere(['status'=>1])
                    ->andWhere(['order_option_id'=>4])
                    ->andWhere(['like', 'created_on', date('Y-m-d') . '%', false])
                    ->one($con);

                    $con->close();

                 




            array_push($allorderlit,array('orderCount'=> $vendorViseOrder['count'] , 'vendorName'=>$allVendor->name ,'dailyOrderCount'=>$vendorViseOrderDaily['count'] , 'kioskOrderDaily'=>$kioskOrderDaily['count'],'posOrderDaily'=>$posOrderDaily['count'],'mobileOrderDaily'=>$mobileOrderDaily['count'],'webOrderDaily'=>$webOrderDaily['count']) );
        
                              $allVendorOrderCount=$allVendorOrderCount+ $vendorViseOrder['count'];         

                            }
                            if(Yii::$app->user->isSuperadmin){

                               
                          $allCount=$allVendorOrderCount;
                            }else{
                                $allCount=$OrderCount;
                            }


                         



        return $this->render('index',[
            'countuser'=>$countuser,
            'countSystemUser'=>$countSystemUser,
            'countVendor'=>$countVendor,
            'allCount'=>$allCount,
            'allorderlit'=>$allorderlit

        ]);
       

    }

    public function actionSetOrderType()
    {
        $type=$_POST['type_id'];
        return $type;

        

    }

    public function actionSalesReportExl()
    {
        
        // $paymentMethod=
        // $vendor=$_POST['vendor'];
        // $start_date=$_POST['start_date'];
        // $paymentMethod=$_POST['paymentMethod'];
        $session = Yii::$app->session;            

        $model=new Vendor();
        $modelPayTypes2=new PayTypes();

 if(Yii::$app->user->isSuperadmin){
    $allOrder=[];
    $vendor_id=$_POST['vendor'];
    $paymethod_id=$_POST['paymentMethod'];
    $start_day=$_POST['start_date'];
    $end_day=$_POST['end_date'];

            if(! $vendor_id==null ){

                $modelVendor=Vendor::find()->where(['id'=>$vendor_id])->one();
                $con = $this->setDb($modelVendor->database_name);
               

                $Orderdate= (new \yii\db\Query())
                ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                ->from('order')
                ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                ->andWhere(['status'=>1])
                ->andFilterWhere(['pay_type_id'=>$paymethod_id])
                ->all($con);
                
                
                $con->close();

                foreach ($Orderdate as $Orderdate) {

                    $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$Orderdate['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$Orderdate['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$Orderdate['order_option_id']])
                        ->one($con);
                        $con->close();
                    // array_push($allOrder,$Orderdate);

                array_push($allOrder,array('AllDate'=> $Orderdate ,'PayTypes_id'=>$modelPayTypes['id'] , 'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );

                
                                                }


                                                return $this->render('sales-report_xlsx',[
                                                    'model'=>$model,
                                                    'allOrder'=>$allOrder,
                                                    'modelPayTypes2'=>$modelPayTypes2,
                                                    'vendor_id'=>$vendor_id,
                                                    'paymethod_id'=>$paymethod_id,
                                                    'start_day'=>$start_day,
                                                    'end_day'=>$end_day,


                                        
                                                ]);
            }

            if($vendor_id==null ){
                $allVendor=Vendor::find()->all();
               

            foreach ($allVendor as $key => $allVendor) {  
                        $con = $this->setDb($allVendor->database_name);

                        $vendorViseOrder = (new \yii\db\Query())
                        ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                        ->from('order')
                        ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                        ->andWhere(['status'=>1])
                        ->andFilterWhere(['pay_type_id'=>$paymethod_id])

                        ->all($con);
                        
                        
                        $con->close();


            foreach ($vendorViseOrder as $vendorViseOrder) {


                $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$vendorViseOrder['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$vendorViseOrder['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$vendorViseOrder['order_option_id']])
                        ->one($con);
                        $con->close();



                array_push($allOrder,array('AllDate'=> $vendorViseOrder,'PayTypes_id'=>$modelPayTypes['id']  , 'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );
            
                                            }

                                }


                                // print_r($allOrder);
                                // dei;



                                return $this->render('sales-report_xlsx',[
                                    'model'=>$model,
                                    'allOrder'=>$allOrder,
                                    'modelPayTypes2'=>$modelPayTypes2,
                                    'vendor_id'=>$vendor_id,
                                    'paymethod_id'=>$paymethod_id,
                                    'start_day'=>$start_day,
                                    'end_day'=>$end_day,
                        
                                ]);

            }

        }else{

            $allOrder=[];
            $allOrder=[];
    // $vendor_id=$_POST['vendor'];
    $paymethod_id=$_POST['paymentMethod'];
    $start_day=$_POST['start_date'];
    $end_day=$_POST['end_date'];




            $db_name = $session->get('vendor_db_name');

  

                

                $modelVendor=Vendor::find()->where(['database_name'=>$db_name])->one();
                $con = $this->setDb($modelVendor->database_name);
               

                $Orderdate= (new \yii\db\Query())
                ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                ->from('order')
                ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                ->andWhere(['status'=>1])
                ->andFilterWhere(['pay_type_id'=>$paymethod_id])
                ->all($con);
                
                
                $con->close();

                foreach ($Orderdate as $Orderdate) {

                    $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$Orderdate['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$Orderdate['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$Orderdate['order_option_id']])
                        ->one($con);
                        $con->close();
                    // array_push($allOrder,$Orderdate);

                array_push($allOrder,array('AllDate'=> $Orderdate,'PayTypes_id'=>$modelPayTypes['id']  , 'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );

                
                                                }


                                                return $this->render('sales-report_xlsx',[
                                                    'model'=>$model,
                                                    'allOrder'=>$allOrder,
                                                    'modelPayTypes2'=>$modelPayTypes2,
                                                    //  'modelVendor'=>$modelVendor,
                                                    'paymethod_id'=>$paymethod_id,
                                                    'start_day'=>$start_day,
                                                    'end_day'=>$end_day,


                                        
                                                ]);
            


        }



        





        return $this->render('sales-report_xlsx');

    }
    
    public function actionAllOrderView()
    {
        $type=$_POST['order_id'];
        $allorderlit=[];
        $session = Yii::$app->session;

        $db_name = $session->get('vendor_db_name');
        $modelvendor= Vendor::find()->where(['database_name'=>$db_name])->one();
    
        $allOrderDine=Order::find()->where(['print_status'=>0])->andWhere(['status'=>1])->andWhere(['order_type'=>2])->andWhere(['view_status'=>0])->all();
        $allOrderTake=Order::find()->where(['print_status'=>0])->andWhere(['status'=>1])->andWhere(['order_type'=>1])->andWhere(['view_status'=>0])->all();
        
        array_push($allorderlit,array('allOrderDine'=>  $allOrderDine,'allOrderTake'=>$allOrderTake));
        return $allorderlit;

        

    }
    public function actionOrderdetails()
    {
        $order_id=$_POST['order_id'];
        // $modelOrder=Order::find()->where(['id'=>$order_id])->one();
        // $modelOrderItem=OrderItem::find()->where(['order_id'=>$order_id])->all();


        // $modelOrderItem  = (new \yii\db\Query())
        // ->select('product.name')
        // ->from('order_item') 
        // // ->innerJoin('product', 'order_item.product_id = product.id')
        // ->where(['order_id'=>$order_id])
        // ->all();

        $OrderDetails=[];
        $session = Yii::$app->session;

        $db_name = $session->get('vendor_db_name');
        $con = $this->setDb($db_name);
        $modelOrderItem  = (new \yii\db\Query())
        ->select('product.name,order_item.quantity')
       ->innerJoin('product', 'order_item.product_id = product.id')
        ->from('order_item') 
        ->where(['order_id'=>$order_id])
        ->all($con);
        $modelOrder = (new \yii\db\Query())
        ->select('*')
        ->from('order') 
        ->where(['id'=>$order_id])
        ->all($con);
        $con->close();


        $modelGenieGet = (new \yii\db\Query())
        ->select('*')
        ->from('genie_get_transaction') 
        ->where(['id'=>$modelOrder[0]['refference_id']])
        ->all();


        $modelUser = (new \yii\db\Query())
        ->select('*')
        ->from('user') 
        ->where(['id'=>$modelOrder[0]['customer_id']])
        ->all();




        array_push($OrderDetails,array('orderProduct'=>  $modelOrderItem,'orderDetails'=>$modelOrder,'modelGenieGet'=>$modelGenieGet ,'modelUser'=>$modelUser));
        return $OrderDetails;
    }


    public function actionOrderNotification()
    {
        $type=$_POST['order_id'];

        $allOrderDine=Order::find()->where(['print_status'=>0])->andWhere(['status'=>1])->andWhere(['order_type'=>2])->andWhere(['view_status'=>0])->all();
        $orderCount=count($allOrderDine);

        return $orderCount;

    }
    public function actionOrderdelivered()
    {
        $order_id=$_POST['order_id'];

        $modelOrder=Order::find()->where(['id'=>$order_id])->one();
        $modelOrder->view_status=1;
       
        if( $modelOrder->save()){
            return true;
        }else{
            return false;

        }
        

        

    }


    public function actionMovingFood()
    {
        if(isset($_POST['submit'])){
    $allOrder=[];
    $start_day=$_POST['start_date'];
    $end_day=$_POST['end_date'];

    // print_r($start_day);
    // print_r($end_day);
    // die;
    $allVendor=Vendor::find()->all();
               

    foreach ($allVendor as $key => $allVendor) {  
                $con = $this->setDb($allVendor->database_name);

                $vendorViseOrder = (new \yii\db\Query())
                ->select('order.refference_id,order.id')
                ->from('order')
                ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                ->andWhere(['status'=>[0,1]])
                // ->andFilterWhere(['pay_type_id'=>$paymethod_id])

                ->all($con);
                
                
                $con->close();

                


    foreach ($vendorViseOrder as $vendorViseOrder) {


        $modelPayTypes = (new \yii\db\Query())
                ->select('order_item.id,order_item.product_id')
                ->from('order_item')
                ->where(['id'=>$vendorViseOrder['id']])
                ->all($con);

        // $modelOrderType = (new \yii\db\Query())
        //         ->select('*')
        //         ->from('order_type')
        //         ->where(['id'=>$vendorViseOrder['order_type']])
        //         ->one($con);

        //         $modelOrderOption= (new \yii\db\Query())
        //         ->select('*')
        //         ->from('order_option')
        //         ->where(['id'=>$vendorViseOrder['order_option_id']])
        //         ->one($con);
        //         $con->close();
        print_r($modelPayTypes);

    if(!$modelPayTypes==null){
        array_push($allOrder,array('AllDate'=> $modelPayTypes) );
    }

       
    
                                    }

                        }

die;
                        






        }
       
        return $this->render('moving-food',[
           

        ]);
    }

    


    public function actionOrderView()
    {
        $session = Yii::$app->session;

        $db_name = $session->get('vendor_db_name');
        $modelvendor= Vendor::find()->where(['database_name'=>$db_name])->one();
    
        $allOrderDine=Order::find()->where(['print_status'=>0])->andWhere(['status'=>1])->andWhere(['order_type'=>2])->andWhere(['view_status'=>0])->all();
        $allOrderTake=Order::find()->where(['print_status'=>0])->andWhere(['status'=>1])->andWhere(['order_type'=>1])->andWhere(['view_status'=>0])->all();
        $this->layout = false;  
        // $this->layout='/main';
        return $this->render('order-view',[
            'allOrderDine'=>$allOrderDine,
            'allOrderTake'=>$allOrderTake,
            'modelvendor'=>$modelvendor,
            

        ]);


    }

    public function actionSalesProductReport()
    {

        $session = Yii::$app->session;
        $db_name = $session->get('vendor_db_name');

        // print_r(date('Y-m-d h:i:s'));
        // die;

  

                

        $modelVendor=Vendor::find()->where(['database_name'=>$db_name])->one();
        $con = $this->setDb($modelVendor->database_name);
      
        $posts =$con->createCommand('SELECT  product.name, order_item.product_id ,sum(order_item.quantity) as prot
        FROM order_item 
        INNER JOIN product
        on product.id = order_item.product_id
        INNER JOIN `order`
        on order.id=order_item.order_id 
        WHERE `order`.status = 1 AND order_item.created_date LIKE "2020-02-13%"
        GROUP by order_item.product_id ')
            ->queryAll();


            return $this->render('sales-product-report',[
                'allOrder'=>$posts,
                'modelVendor'=>$modelVendor
            
    
    
            ]);


    }


    public function actionSalesProductReportExl()
    {


        $session = Yii::$app->session;
        $db_name = $session->get('vendor_db_name');

        // print_r(date('Y-m-d h:i:s'));
        // die;

  

                

        $modelVendor=Vendor::find()->where(['database_name'=>$db_name])->one();
        $con = $this->setDb($modelVendor->database_name);
      
        $posts =$con->createCommand('SELECT  product.name, order_item.product_id ,sum(order_item.quantity) as prot
        FROM order_item 
        INNER JOIN product
        on product.id = order_item.product_id
        INNER JOIN `order`
        on order.id=order_item.order_id 
        WHERE `order`.status = 1 AND order_item.created_date LIKE "2020-02-13%"
        GROUP by order_item.product_id ')
            ->queryAll();


            return $this->render('sales-product-report-xlsx',[
                'allOrder'=>$posts,
                'modelVendor'=>$modelVendor
            
    
    
            ]);


    }




    public function actionSalesReport()
    {
        $model=new Vendor();
        $modelPayTypes2=new PayTypes();
        $session = Yii::$app->session;            

        if(isset($_POST['submit'])){
           
            // print_r($paymethod_id);
            // die;

 if(Yii::$app->user->isSuperadmin){
    $allOrder=[];
    $vendor_id=$_POST['Vendor']['id'];
    $paymethod_id=$_POST['PayTypes']['id'];
    $start_day=$_POST['start_date'];
    $end_day=$_POST['end_date'];

            if(! $vendor_id==null ){

                $modelVendor=Vendor::find()->where(['id'=>$vendor_id])->one();
                $con = $this->setDb($modelVendor->database_name);
               

                $Orderdate= (new \yii\db\Query())
                ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                ->from('order')
                ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                ->andWhere(['status'=>1])
                ->andFilterWhere(['pay_type_id'=>$paymethod_id])
                ->all($con);
                
                
                $con->close();

                foreach ($Orderdate as $Orderdate) {

                    $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$Orderdate['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$Orderdate['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$Orderdate['order_option_id']])
                        ->one($con);
                        $con->close();
                    // array_push($allOrder,$Orderdate);

                array_push($allOrder,array('AllDate'=> $Orderdate ,'PayTypes_id'=>$modelPayTypes['id'] ,'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );

                
                                                }


                                                return $this->render('sales-report',[
                                                    'model'=>$model,
                                                    'allOrder'=>$allOrder,
                                                    'modelPayTypes2'=>$modelPayTypes2,
                                                    'vendor_id'=>$vendor_id,
                                                    'paymethod_id'=>$paymethod_id,
                                                    'start_day'=>$start_day,
                                                    'end_day'=>$end_day,


                                        
                                                ]);
            }

            if($vendor_id==null ){
                $allVendor=Vendor::find()->all();
               

            foreach ($allVendor as $key => $allVendor) {  
                        $con = $this->setDb($allVendor->database_name);

                        $vendorViseOrder = (new \yii\db\Query())
                        ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                        ->from('order')
                        ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                        ->andWhere(['status'=>1])
                        ->andFilterWhere(['pay_type_id'=>$paymethod_id])

                        ->all($con);
                        
                        
                        $con->close();


            foreach ($vendorViseOrder as $vendorViseOrder) {


                $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$vendorViseOrder['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$vendorViseOrder['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$vendorViseOrder['order_option_id']])
                        ->one($con);
                        $con->close();



                array_push($allOrder,array('AllDate'=> $vendorViseOrder , 'PayTypes_id'=>$modelPayTypes['id'] ,'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );
            
                                            }

                                }


                                // print_r($allOrder);
                                // dei;



                                return $this->render('sales-report',[
                                    'model'=>$model,
                                    'allOrder'=>$allOrder,
                                    'modelPayTypes2'=>$modelPayTypes2,
                                    'vendor_id'=>$vendor_id,
                                    'paymethod_id'=>$paymethod_id,
                                    'start_day'=>$start_day,
                                    'end_day'=>$end_day,
                        
                                ]);

            }

        }else{

            $allOrder=[];
            // $vendor_id=$_POST['Vendor']['id'];
            $paymethod_id=$_POST['PayTypes']['id'];
            $start_day=$_POST['start_date'];
            $end_day=$_POST['end_date'];



            $db_name = $session->get('vendor_db_name');

  

                

                $modelVendor=Vendor::find()->where(['database_name'=>$db_name])->one();
                $con = $this->setDb($modelVendor->database_name);
               

                $Orderdate= (new \yii\db\Query())
                ->select('order.refference_id,order.number_of_items,order.total_amount,order.created_on,order.created_by,order.pay_type_id,order.order_type,order.order_option_id,order.full_amount')
                ->from('order')
                ->FilterWhere(['between', 'created_on',  $start_day.' 00:00:00',  $end_day.' 23:59:59' ])
                ->andWhere(['status'=>1])
                ->andFilterWhere(['pay_type_id'=>$paymethod_id])
                ->all($con);
                
                
                $con->close();

                foreach ($Orderdate as $Orderdate) {

                    $modelPayTypes = (new \yii\db\Query())
                        ->select('*')
                        ->from('pay_types')
                        ->where(['id'=>$Orderdate['pay_type_id']])
                        ->one($con);

                $modelOrderType = (new \yii\db\Query())
                        ->select('*')
                        ->from('order_type')
                        ->where(['id'=>$Orderdate['order_type']])
                        ->one($con);

                        $modelOrderOption= (new \yii\db\Query())
                        ->select('*')
                        ->from('order_option')
                        ->where(['id'=>$Orderdate['order_option_id']])
                        ->one($con);
                        $con->close();
                    // array_push($allOrder,$Orderdate);

                array_push($allOrder,array('AllDate'=> $Orderdate ,'PayTypes_id'=>$modelPayTypes['id'] , 'PayTypes'=>$modelPayTypes['name'],'OrderType'=>$modelOrderType['name'],'OrderOption'=>$modelOrderOption['name']) );

                
                                                }


                                                return $this->render('sales-report',[
                                                    'model'=>$model,
                                                    'allOrder'=>$allOrder,
                                                    'modelPayTypes2'=>$modelPayTypes2,
                                                    //  'modelVendor'=>$modelVendor,
                                                    'paymethod_id'=>$paymethod_id,
                                                    'start_day'=>$start_day,
                                                    'end_day'=>$end_day,


                                        
                                                ]);
            


        }



        }
        return $this->render('sales-report',[
            'model'=>$model,
            'modelPayTypes2'=>$modelPayTypes2,
            'vendor_id'=>"",
            'paymethod_id'=>"",
             'start_day'=>" ",
             'end_day'=>" ",


        ]);
    }

    /**
     * Login action.
     *
     * @return string
     */
    public function actionLogin()
    {
        if (!Yii::$app->user->isGuest) {
            return $this->goHome();
        }

        $model = new LoginForm();
        if ($model->load(Yii::$app->request->post()) && $model->login()) {
            return $this->goBack();
        } else {
            $model->password = '';

            return $this->render('login', [
                'model' => $model,
            ]);
        }
    }

    /**
     * Logout action.
     *
     * @return string
     */
    public function actionLogout()
    {
        Yii::$app->user->logout();

        return $this->goHome();
    }

    public function beforeAction($action) 
    { 
        $this->enableCsrfValidation = false; 
        return parent::beforeAction($action); 
    } 
}
