<?php

namespace backend\controllers;

use Yii;
use backend\models\Vendor;
use backend\models\Order;
use backend\models\Organization;
use backend\models\VendorSearch;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use yii\filters\ContentNegotiator;
use yii\web\Response;
use yii\filters\AccessControl;


/**
 * VendorController implements the CRUD actions for Vendor model.
 */
class VendorController extends Controller
{
    /**
     * {@inheritdoc}
     */
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'delete' => ['POST'],
                    'shopclose'=>['POST'],
                    'shopopne'=>['POST'],
                ],
            ],
            'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'actions' => ['create', 'view','index','update','shopclose','shopopne' ],
                        'allow' => true,
                        'roles' => ['@'],
                    ],
                ],
            ],
            'contentNegotiator' => [
                'class' => ContentNegotiator::className(),
                'only' => ['shopclose','shopopne'], 
                'formats' => [
                    'application/json' => Response::FORMAT_JSON,
                ],
            ]
        ];
    }

    /**
     * Lists all Vendor models.
     * @return mixed
     */
    public function actionIndex()
    {
        $searchModel = new VendorSearch();
        $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

        return $this->render('index', [
            'searchModel' => $searchModel,
            'dataProvider' => $dataProvider,
        ]);
    }


    public function actionShopclose() { 
            
        $id = $_POST['vendor_id'];
       $model_vandor=Vendor::find()->where(['id'=>$id])->one();
       $model_vandor->status=0;
       $model_vandor->save();

      
        return $id;
    } 

    public function actionShopopne() { 
            
        $id = $_POST['vendor_id'];
       $model_vandor=Vendor::find()->where(['id'=>$id])->one();
       $model_vandor->status=1;
       $model_vandor->save();

      
        return $id;
    } 

    /**
     * Displays a single Vendor model.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionView($id)
    {
        $VendorModel=$this->findModel($id);
        $con = Yii::$app->PosDbManager->getDbConnection($VendorModel->database_name);



        $orderModelcount= Order::find()->all($con);
        $orderModel= Order::find()->limit(5)->orderBy(['id' => SORT_DESC])->all($con);
         $ordercount =count($orderModelcount);
        
        return $this->render('view', [
            'model' => $this->findModel($id),
            'ordercount'=>$ordercount,
            'orderModel'=>$orderModel
        ]);
    }

    /**
     * Creates a new Vendor model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     * @return mixed
     */
    public function actionCreate()
    {
        $model = new Vendor();

        if ($model->load(Yii::$app->request->post())) {
            $model->created=date('Y-m-d h:i:s');


            // vendor database create startddddddfd

            $organization_code=Organization::find()->where(['id'=>$model->organization_id])->one()->code;

            



            $databases_name="s_pos_".$organization_code."_".$model->code;
            

            $result = Yii::$app->db->createCommand('CREATE DATABASE '.$databases_name.'')->execute();
            
            $connection = new \yii\db\Connection([
                'dsn' => 'mysql:host=localhost;dbname='.$databases_name.'',
            'username' => 'eimuser',
            'password' => 'Welcome1',
            'charset' => 'utf8',
            ]);
            $connection->open();
            $sql = file_get_contents('../database/s_pos_structure_new2.sql');
            $connection->createCommand($sql)->execute();
            $connection->close();








           
            $new_video_path = '../image/'.$databases_name.'';
            //create a new dir
            mkdir($new_video_path); 

            $model->database_name=$databases_name;
            $model->status=1;
            $model->save();

            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('create', [
            'model' => $model,
        ]);
    }

    /**
     * Updates an existing Vendor model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionUpdate($id)
    {
        $model = $this->findModel($id);

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            return $this->redirect(['view', 'id' => $model->id]);
        }

        return $this->render('update', [
            'model' => $model,
        ]);
    }

    /**
     * Deletes an existing Vendor model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id
     * @return mixed
     * @throws NotFoundHttpException if the model cannot be found
     */
    public function actionDelete($id)
    {
        $this->findModel($id)->delete();

        return $this->redirect(['index']);
    }

    /**
     * Finds the Vendor model based on its primary key value.
     * If the model is not found, a 404 HTTP exception will be thrown.
     * @param integer $id
     * @return Vendor the loaded model
     * @throws NotFoundHttpException if the model cannot be found
     */
    protected function findModel($id)
    {
        if (($model = Vendor::findOne($id)) !== null) {
            return $model;
        }

        throw new NotFoundHttpException('The requested page does not exist.');
    }

    public function beforeAction($action) 
    { 
        $this->enableCsrfValidation = false; 
        return parent::beforeAction($action); 
    } 
}
