<?php

namespace backend\models;

use Yii;
use backend\models\Vendor;
use backend\models\PriceList;
use yii\helpers\ArrayHelper;




/**
 * This is the model class for table "product".
 *
 * @property int $id
 * @property string $name
 * @property int $available_quantity
 * @property double  $rating
 * @property int $status
 * @property string $image_path
 * @property string $details_image_path
 * @property int $sub_category_id
 * @property int $main_category_id
 * @property int $uom_id
 *
 * @property MenuTypeItems[] $menuTypeItems
 * @property OrderItem[] $orderItems
 * @property PriceList[] $priceLists
 * @property MainCategory $mainCategory
 * @property SubCategory $subCategory
 * @property MunuType $menuType
 */
class Product extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'product';
    }
    // public $vendor_id;
    // public $organization_id;
    public $file;
    public $file2;
    public $selling_price;
    public $buying_price;
  


    /**
     * @return \yii\db\Connection the database connection used by this AR class.
     */
    public static function getDb()
    {

        $session = Yii::$app->session;

        $db_name = $session->get('vendor_db_name');

        if(! $session->get($db_name) ){

            $connection =Yii::$app->PosDbManager->getDbConnection($db_name);
            $dbcon= $session->set($db_name,  $connection);        
            return $connection;

        }else{
            $dbcon= $session->get($db_name);       

            return $dbcon;
        }
      
    }
    public $product_id;

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'available_quantity', 'rating', 'status', 'details_image_path', 'munu_type_id','sub_category_id', 'main_category_id', 'uom_id', 'created', 'created_by'], 'required'],
            [['available_quantity', 'status', 'sub_category_id', 'main_category_id', 'munu_type_id', 'uom_id', 'created_by'], 'integer'],
            [['rating'], 'number'],
            [['created'], 'safe'],
            ['file', 'file', 'extensions' => 'jpg, png'], 
            ['file2', 'file', 'extensions' => 'jpg, png'], 
            [['name', 'image_path', 'details_image_path'], 'string', 'max' => 250],
            [['main_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => MainCategory::className(), 'targetAttribute' => ['main_category_id' => 'id']],
             [['sub_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => SubCategory::className(), 'targetAttribute' => ['sub_category_id' => 'id']],
             [['munu_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => MenuType::className(), 'targetAttribute' => ['munu_type_id' => 'id']], 
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'available_quantity' => 'Available Quantity',
            'rating' => 'Rating',
            'status' => 'Status',
            'image_path' => 'Image Path',
            'details_image_path' => 'Details Image Path',
            'sub_category_id' => 'Sub Category',
            'main_category_id' => 'Main Category',
            'munu_type_id'=>'Menu Type',
            'uom_id' => 'Uom ',
            'created' => 'Created',
            'created_by' => 'Created By',
            'file'=>'Image Path',
            'file2'=>'Details Image Path',
            'selling_price'=>'Selling Price',
            'buying_price'=> 'Buying Price'
           

           
        ];
    }



    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMenuTypeItems()
    {
        return $this->hasMany(MenuTypeItems::className(), ['product_id' => 'id']);
    }
            // 'vendor_id'=>'Vendor Name',
            // 'organization_id'=>'Organization Name',
            
  
 

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrderItems()
    {
        return $this->hasMany(OrderItem::className(), ['product_id' => 'id']);
    }


     /**
     * @return \yii\db\ActiveQuery
     */
    public function getPriceLists0()
    {
         return $this->hasOne(PriceList::className(), ['product_id' => 'id'])->alias('priceLists0');;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPriceLists()
    {
        $price= PriceList:: find()->where(['product_id' => 'id' , 'is_active' => 1])->one() ;
        //return $this->hasOne(PriceList::className(), ['product_id' => 'id' , 'is_active' => 1]);

        return $price;
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMainCategory()
    {
        return $this->hasOne(MainCategory::className(), ['id' => 'main_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubCategory()
    {
        return $this->hasOne(SubCategory::className(), ['id' => 'sub_category_id']);
    }


    /** 
    * @return \yii\db\ActiveQuery 
    */ 
   public function getMunuType() 
   { 
       return $this->hasOne(MenuType::className(), ['id' => 'munu_type_id']); 
   } 
   
    /** 
    * @return \yii\db\ActiveQuery 
    */ 
    public function getUom() 
    { 
        return $this->hasOne(Uom::className(), ['id' => 'uom_id']); 
    } 

    public static function getList()
    {
       
            return array('' => '') + ArrayHelper::map(self::find()->all(), 'id', 'name');
       
    }
}
