<?php

namespace backend\models;

use yii\base\Model;
use yii\data\ActiveDataProvider;
use backend\models\Product;
use backend\models\MainCategory;
use backend\models\MenuType;
use yii\db\Query;


/**
 * ProductSearch represents the model behind the search form of `backend\models\Product`.
 */
class ProductSearch extends Product
{
    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['id', 'available_quantity', 'status', 'sub_category_id', 'munu_type_id', 'uom_id', 'created_by'], 'integer'],
           [['name', 'image_path', 'details_image_path', 'created','main_category_id','selling_price','buying_price'], 'safe'],
           [['rating'], 'number'],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function scenarios()
    {
        // bypass scenarios() implementation in the parent class
        return Model::scenarios();
    }

    /**
     * Creates data provider instance with search query applied
     *
     * @param array $params
     *
     * @return ActiveDataProvider
     */
    public function search($params)
    {
        // $query = Product::find();


        $query = new Query();
$query= Product::find()     
->select(['price_list.selling_price','price_list.buying_price','product.*'])  
    ->from('product')
    ->join(' LEFT OUTER JOIN', 'price_list',
        'price_list.product_id = product.id')
        ->where(['=', 'price_list.is_active', 1]);

        // add conditions that should always apply here

        // print_r($query);
        // die;

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'sort' => ['attributes' => ['id','name','available_quantity','rating','munu_type_id','sub_category_id','main_category_id','uom_id','selling_price','status','buying_price']],

            'pagination' => [
                'pageSize' => 23,],
        ]);

        $this->load($params);

        if (!$this->validate()) {
            // uncomment the following line if you do not want to return any records when validation fails
            // $query->where('0=1');
            return $dataProvider;
        }


        $query->joinWith('priceLists0');


       

        // grid filtering conditions
        $query->andFilterWhere([
            'id' => $this->id,
            'available_quantity' => $this->available_quantity,
            'rating' => $this->rating,
            'status' => $this->status,
            'sub_category_id' => $this->sub_category_id,
            'main_category_id' => $this->main_category_id,
            'munu_type_id'=> $this->munu_type_id,
            'uom_id' => $this->uom_id,
        ]);

        $query->andFilterWhere(['like', 'name', $this->name])
            ->andFilterWhere(['like', 'image_path', $this->image_path])
            ->andFilterWhere(['like', 'price_list.selling_price', $this->selling_price])
            ->andFilterWhere(['like', 'price_list.buying_price', $this->buying_price])
            ->andFilterWhere(['like', 'details_image_path', $this->details_image_path]);    


            // print_r( $dataProvider);
            // die;

        return $dataProvider;
    }
}
