<?php
namespace app\modules\api\controllers;

use Yii;
use yii\web\Controller;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use yii\filters\ContentNegotiator;
use yii\web\Response;
use webvimark\modules\UserManagement\models\User;
use webvimark\modules\UserManagement\models\forms\LoginForm;
use yii\helpers\Json;
use webvimark\modules\UserManagement\components\UserIdentity;
use app\models\Userinstitute;
use app\models\Course;
use app\models\TeacherUser;

class LoginController extends Controller
{

    public $enableCsrfValidation = false;
    public $freeAccessActions = ['login'];

	public function behaviors()
	{
		return [
			'verbs' => [
				'class' => VerbFilter::className(),
		        'actions' => [
                    //'allow' => true,
                    'index' => ['post'],
                    'user-institutes' => ['post']
                    //'roles' => ['*'],
		        ]
			],
			'access' => [
                'class' => AccessControl::className(),
                'rules' => [
                    [
                        'allow' => true,
						'actions' => ['login','user-institutes'],
                        //'roles' => ['*'],
                    ],
                ],
            ],
			'contentNegotiator' => [
	            'class' => ContentNegotiator::className(),
	            'formats' => [
	                'application/json' => Response::FORMAT_JSON,
	            ],
	        ]
		];
    }
    
    public function actionLogin()
    {
        $request = Yii::$app->request;
        if ($request->isPost) 
        {
            $post = Json::decode($request->rawBody);
          
            $model = new LoginForm();
            $model->username = $post['username'];
            $model->password = $post['password'];
            $instituteid = $post['instituteid'];
            $deviceid = $post['deviceid'];
            
            if(!$model->login()){
                return [
                    'success' => 0,
                    'message' => 'Incorrect Username or Password'
                ];
            }else{
                $user = $model->getUser();
                $instituteCourse = Course::find()->where(['Institute_id' => $instituteid])->all();
                $teacheruser = TeacherUser::find()->where(['user_id' => $user->id])->one();
                if(!empty($teacheruser)){
                    $teacher = $teacheruser->teacher;
                    $teacherCourses = Course::find()->where(['Institute_id' => $instituteid, "Teacher_id"=>$teacher->id ])->all();
                }else{
                    $teacherCourses = json_decode ("{}");
                }
                
                return [
                    'success' => 1,
                    'user' =>  [
                        "id" => $user->id,
                        "username" => $user->username,
                        "auth_key" => $user->auth_key,
                        "status" => $user->status,
                        "superadmin" => $user->superadmin,
                        "email" => $user->email,
                        "institute_courses" => $instituteCourse,
                        "teacher" => empty($teacher) ? json_decode ("{}") : [
                            "id" => $teacher->id,
                            "firstName" => $teacher->firstName,
                            "lastName" => $teacher->lastName,
                            "email" => $teacher->email,
                            "contactNumber" => $teacher->contactNumber,
                            "nic" => $teacher->nic,
                            "teacher_courses" =>$teacherCourses,
                        ]
                    ]
                ];
            }
        }
    }

    public function actionUserInstitutes()
    {
        $request = Yii::$app->request;
        if ($request->isPost) 
        {
            $post = Json::decode($request->rawBody);
          
            $username = $post['username'];
            $user = User::findByUsername($username);
            $institutesArr = Userinstitute::find()->where(['user_id' => $user->id])->all();
           
            $response = [];

            foreach($institutesArr as $u_ins){
                $response[] = $u_ins->institute;
            }
            return $response;
            
        }
    }
}