<?php

namespace frontend\models;

use Yii;
use backend\models\Vendor;
use frontend\models\OrderSerial;
use yii\data\Pagination;
/**
 * This is the model class for table "order".
 *
 * @property int $id
 * @property string $refference_id
 * @property int $number_of_items
 * @property string $total_amount
 * @property int $status
 * @property string $created_on
 * @property int $created_by
 * @property int $customer_id
 * @property int $rating
 * @property int $pay_type_id
 *
 * @property PayTypes $payType
 * @property OrderItem[] $orderItems
 */
class Order extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'order';
    }

    /**
     * @return \yii\db\Connection the database connection used by this AR class.
     */
    public static function getDb()
    {
        $session = Yii::$app->session;
        $db_name = $session->get('vendor_db_name');
        if(! $session->get($db_name) ){

            $connection =Yii::$app->PosDbManager->getDbConnection($db_name);

            
            
         $dbcon= $session->set($db_name, $connection);
        
         return $connection;
        }else{
            $dbcon= $session->get($db_name);
          
            return $dbcon;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['refference_id', 'number_of_items', 'total_amount', 'status', 'created_on', 'created_by', 'customer_id', 'rating', 'order_type'], 'required'],
            [['number_of_items', 'status', 'created_by', 'customer_id', 'rating', 'pay_type_id', 'order_type', 'order_option_id','print_status','view_status'], 'integer'],
            [['total_amount','full_amount','discount'], 'number'],
            [['created_on'], 'safe'],
            [['customer_req'], 'string'], 

            [['refference_id'], 'string', 'max' => 150],
            [['pay_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => PayTypes::className(), 'targetAttribute' => ['pay_type_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'refference_id' => 'Refference ID',
            'number_of_items' => 'Number Of Items',
            'total_amount' => 'Total Amount',
            'status' => 'Status',
            'created_on' => 'Created On',
            'created_by' => 'Created By',
            'customer_id' => 'Customer ID',
            'rating' => 'Rating',
            'pay_type_id' => 'Pay Type ID',
            'full_amount'=>'Full Amount',
            'customer_req'=>'Customer Req',
            'view_status'=>'View Status',
            'discount'=>'Discount'
        ];
    }


      /**
    * @return \yii\db\ActiveQuery
    */
   public function getOrderType() 
   { 
       return $this->hasOne(OrderType::className(), ['id' => 'order_type']); 
   } 
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPayType()
    {
        return $this->hasOne(PayTypes::className(), ['id' => 'pay_type_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrderItems()
    {
        return $this->hasMany(OrderItem::className(), ['order_id' => 'id']);
    }

    public function getOrderOption() 
    { 
        return $this->hasOne(OrderOption::className(), ['id' => 'order_option_id']); 
    } 

     /**
     * @return int
     */
    public function getCustomerOrderCount($dbName,$userId)
    {   
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);

        $ordersCount = (new \yii\db\Query())
                    ->select('COUNT(*)')
                    ->from('order') 
                    ->where(['customer_id' => $userId])
                    ->one($con); 

        $con -> close();

        return $ordersCount['COUNT(*)'];
    }

    

     /**
     * @return arrray()
     */
    public function getCustomerOrders($dbName,$userId,$pages)
    {
      
        $orders = array();
       
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);
       
        $orders  = Order::find()->where(['customer_id' =>  $userId])
                        ->offset($pages->offset)
                        ->limit($pages->limit)
                        ->orderBy([
                            'id' => SORT_DESC,
                            // 'item_no'=>SORT_ASC
                          ])
                        ->all($con);

        $con -> close();

       
       
        return $orders;
    }

     /**
     * @return arrray()
     */
    public function getAllCustomerOrders($dbName,$userId)
    {
      
        $orders = array();
       
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);
       
        $orders  = Order::find()->where(['customer_id' =>  $userId])->all($con);

        $con -> close();
       
        return $orders;
    }


    /**
     * @return arrray()
     */
    public function searchCustomerOrders($dbName, $userId, $orderRefference, $limit)
    {
        $orders = array();
        
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);

        $orders  = Order::find()->where(['customer_id' => $userId])
                    ->andWhere(['like', 'refference_id', $orderRefference])
                    ->limit($limit)
                    ->all($con);

        $con -> close();

        return $orders;
    }

    /**
     * @return arrray()
     */
    public function FilterCustomerOrders($dbName, $userId, $rating, $limit)
    {
        $orders = array();
        
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);

        $orders  = Order::find()
                    ->where(
                            [
                                'customer_id' => $userId,
                                'rating' => $rating
                            ])
                    ->orderBy(['created_on'=>SORT_DESC])
                    ->limit($limit)
                    ->all($con);

        $con -> close();

        return $orders;
    }

    /**
     * @return arrray()
     */
    public function getVendorsByCustomerOrders()
    {
        return Vendor::find()->where(['status'=>1])->all();
    }

    /**
     * @return Vendor
     */
    public function getSelectedVendor($vendorId)
    {
        $session = Yii::$app->session;
           
        
         if($vendorId == null){
            $db_name = $session->get('select_vandor');
             if($db_name){
                $vendorId = $db_name;
             }else{
                $vendorId =3;
             }

         }
             

         return Vendor::find() ->where(['id' => $vendorId])->one();
    }

    /**
     * @return Pagination
     */
    public function getPagination($totalCount,$pageSize)
    {
       return new Pagination(
            [
                'totalCount' => $totalCount
                ,'pageSize' => $pageSize
            ]);
    }

      /**
     * @return arrray()
     */
    public function getTopOrderedProducts($dbName,$count,$orderIds)
    {
         
        $con = Yii::$app->PosDbManager
        ->getDbConnection($dbName);

        $topProducts  = (new \yii\db\Query())
        ->select([
            'product.*',
            'order_item.unit_price',
            'SUM([[order_item.quantity]]) AS QuentitySum'
            ])
        ->from('order_item') 
        ->innerJoin('product', 'order_item.product_id = product.id')
        ->where(['in', 'order_item.order_id', $orderIds])
        ->groupBy(['product.id'])
        ->orderBy('QuentitySum DESC')
        ->limit($count)
        ->all($con);

        $con -> close();

        return $topProducts;

    }

    public function getSerialNum()
    {
        $lastmodelOrderSerial = OrderSerial::find()->orderBy(['id'=> SORT_DESC])->one();
        $modeOrderSerial=new OrderSerial();

        if($lastmodelOrderSerial==null){
            $modeOrderSerial->serial_num=1;
            $modeOrderSerial->order_id=12;
            $modeOrderSerial->save();

            return $modeOrderSerial->serial_num;
           
        }else{
            $modeOrderSerial->serial_num=$lastmodelOrderSerial->serial_num+1;
            $modeOrderSerial->order_id=12;
            $modeOrderSerial->save();
            return $modeOrderSerial->serial_num;

        }

        
    }


    
}
