<?php

namespace frontend\models;

use Yii;

/**
 * This is the model class for table "product".
 *
 * @property int $id
 * @property string $name
 * @property int $available_quantity
 * @property double $rating
 * @property int $status
 * @property string $image_path
 * @property string $details_image_path
 * @property int $sub_category_id
 * @property int $main_category_id
 * @property int $munu_type_id
 * @property int $uom_id
 * @property string $created
 * @property int $created_by
 *
 * @property Discount[] $discounts
 * @property OrderItem[] $orderItems
 * @property PriceList[] $priceLists
 * @property MainCategory $mainCategory
 * @property SubCategory $subCategory
 * @property MenuType $munuType
 */
class Product extends \yii\db\ActiveRecord
{
    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'product';
    }

    /**
     * @return \yii\db\Connection the database connection used by this AR class.
     */
    public static function getDb()
    {
        $session = Yii::$app->session;
        $db_name = $session->get('vendor_db_name');
        if(! $session->get($db_name) ){
            $connection = new \yii\db\Connection([
                'dsn' => 'mysql:host=localhost;dbname='.$db_name.'',
            'username' => 'eimuser',
            'password' => 'Welcome@2019',
            'charset' => 'utf8',
        
            
            ]);
            
            
         $dbcon= $session->set($db_name, $connection);
        
         return $connection;
        }else{
            $dbcon= $session->get($db_name);
          
            return $dbcon;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['name', 'available_quantity', 'rating', 'status', 'image_path', 'details_image_path', 'sub_category_id', 'main_category_id', 'uom_id', 'created', 'created_by'], 'required'],
            [['available_quantity', 'status', 'sub_category_id', 'main_category_id', 'munu_type_id', 'uom_id', 'created_by'], 'integer'],
            [['rating'], 'number'],
            [['created'], 'safe'],
            [['name', 'image_path', 'details_image_path'], 'string', 'max' => 250],
            [['main_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => MainCategory::className(), 'targetAttribute' => ['main_category_id' => 'id']],
            [['sub_category_id'], 'exist', 'skipOnError' => true, 'targetClass' => SubCategory::className(), 'targetAttribute' => ['sub_category_id' => 'id']],
            [['munu_type_id'], 'exist', 'skipOnError' => true, 'targetClass' => MenuType::className(), 'targetAttribute' => ['munu_type_id' => 'id']],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'name' => 'Name',
            'available_quantity' => 'Available Quantity',
            'rating' => 'Rating',
            'status' => 'Status',
            'image_path' => 'Image Path',
            'details_image_path' => 'Details Image Path',
            'sub_category_id' => 'Sub Category ID',
            'main_category_id' => 'Main Category ID',
            'munu_type_id' => 'Munu Type ID',
            'uom_id' => 'Uom ID',
            'created' => 'Created',
            'created_by' => 'Created By',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDiscounts()
    {
        return $this->hasMany(Discount::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getOrderItems()
    {
        return $this->hasMany(OrderItem::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getPriceLists()
    {
        return $this->hasMany(PriceList::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMainCategory()
    {
        return $this->hasOne(MainCategory::className(), ['id' => 'main_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubCategory()
    {
        return $this->hasOne(SubCategory::className(), ['id' => 'sub_category_id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMunuType()
    {
        return $this->hasOne(MenuType::className(), ['id' => 'munu_type_id']);
    }
}
